@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2011  See the AUTHORS file for details.
+ * Copyright (C) 2004-2012  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,19 +9,15 @@
 #ifndef _CHAN_H
 #define _CHAN_H
 
-#include "zncconfig.h"
-#include "Nick.h"
-#include "ZNCString.h"
+#include <znc/zncconfig.h>
+#include <znc/Nick.h>
+#include <znc/ZNCString.h>
+#include <znc/Buffer.h>
 #include <map>
-#include <set>
-#include <vector>
-
-using std::vector;
-using std::map;
-using std::set;
 
 // Forward Declarations
 class CUser;
+class CIRCNetwork;
 class CClient;
 class CConfig;
 class CFile;
@@ -52,11 +48,11 @@ public:
 		M_Except     = 'e'
 	} EModes;
 
-	CChan(const CString& sName, CUser* pUser, bool bInConfig, CConfig *pConfig = NULL);
+	CChan(const CString& sName, CIRCNetwork* pNetwork, bool bInConfig, CConfig *pConfig = NULL);
 	~CChan();
 
 	void Reset();
-	bool WriteConfig(CFile& File);
+	CConfig ToConfig();
 	void Clone(CChan& chan);
 	void Cycle() const;
 	void JoinUser(bool bForce = false, const CString& sKey = "", CClient* pClient = NULL);
@@ -86,9 +82,11 @@ public:
 	// !Nicks
 
 	// Buffer
-	int AddBuffer(const CString& sLine);
-	void ClearBuffer();
-	void TrimBuffer(const unsigned int uMax);
+	const CBuffer& GetBuffer() const { return m_Buffer; }
+	unsigned int GetBufferCount() const { return m_Buffer.GetLineCount(); }
+	bool SetBufferCount(unsigned int u, bool bForce = false) { return m_Buffer.SetLineCount(u, bForce); };
+	size_t AddBuffer(const CString& sFormat, const CString& sText = "", const timeval* ts = NULL) { return m_Buffer.AddLine(sFormat, sText, ts); }
+	void ClearBuffer() { m_Buffer.Clear(); }
 	void SendBuffer(CClient* pClient);
 	// !Buffer
 
@@ -100,41 +98,40 @@ public:
 	// !wrappers
 
 	// Setters
+	void SetModeKnown(bool b) { m_bModeKnown = b; }
 	void SetIsOn(bool b) { m_bIsOn = b; if (!b) { Reset(); } }
 	void SetKey(const CString& s) { m_sKey = s; }
 	void SetTopic(const CString& s) { m_sTopic = s; }
 	void SetTopicOwner(const CString& s) { m_sTopicOwner = s; }
 	void SetTopicDate(unsigned long u) { m_ulTopicDate = u; }
 	void SetDefaultModes(const CString& s) { m_sDefaultModes = s; }
-	bool SetBufferCount(unsigned int u, bool bForce = false);
-	void SetKeepBuffer(bool b) { m_bKeepBuffer = b; }
+	void SetAutoClearChanBuffer(bool b);
 	void SetDetached(bool b = true) { m_bDetached = b; }
 	void SetInConfig(bool b) { m_bInConfig = b; }
 	void SetCreationDate(unsigned long u) { m_ulCreationDate = u; }
 	void Disable() { m_bDisabled = true; }
-	void Enable() { m_bDisabled = false; }
+	void Enable();
 	void IncJoinTries() { m_uJoinTries++; }
 	void ResetJoinTries() { m_uJoinTries = 0; }
 	// !Setters
 
 	// Getters
+	bool IsModeKnown() const { return m_bModeKnown; }
 	bool HasMode(unsigned char uMode) const;
 	CString GetOptions() const;
 	CString GetModeArg(unsigned char uMode) const;
-	map<char, unsigned int> GetPermCounts() const;
+	std::map<char, unsigned int> GetPermCounts() const;
 	bool IsOn() const { return m_bIsOn; }
 	const CString& GetName() const { return m_sName; }
-	const map<unsigned char, CString>& GetModes() const { return m_musModes; }
+	const std::map<unsigned char, CString>& GetModes() const { return m_musModes; }
 	const CString& GetKey() const { return m_sKey; }
 	const CString& GetTopic() const { return m_sTopic; }
 	const CString& GetTopicOwner() const { return m_sTopicOwner; }
-	unsigned int GetTopicDate() const { return m_ulTopicDate; }
+	unsigned long GetTopicDate() const { return m_ulTopicDate; }
 	const CString& GetDefaultModes() const { return m_sDefaultModes; }
-	const vector<CString>& GetBuffer() const { return m_vsBuffer; }
-	const map<CString,CNick>& GetNicks() const { return m_msNicks; }
-	unsigned int GetNickCount() const { return m_msNicks.size(); }
-	unsigned int GetBufferCount() const { return m_uBufferCount; }
-	bool KeepBuffer() const { return m_bKeepBuffer; }
+	const std::map<CString,CNick>& GetNicks() const { return m_msNicks; }
+	size_t GetNickCount() const { return m_msNicks.size(); }
+	bool AutoClearChanBuffer() const { return m_bAutoClearChanBuffer; }
 	bool IsDetached() const { return m_bDetached; }
 	bool InConfig() const { return m_bInConfig; }
 	unsigned long GetCreationDate() const { return m_ulCreationDate; }
@@ -145,7 +142,7 @@ private:
 protected:
 	bool                         m_bDetached;
 	bool                         m_bIsOn;
-	bool                         m_bKeepBuffer;
+	bool                         m_bAutoClearChanBuffer;
 	bool                         m_bInConfig;
 	bool                         m_bDisabled;
 	CString                      m_sName;
@@ -154,15 +151,15 @@ protected:
 	CString                      m_sTopicOwner;
 	unsigned long                m_ulTopicDate;
 	unsigned long                m_ulCreationDate;
-	CUser*                       m_pUser;
+	CIRCNetwork*                 m_pNetwork;
 	CNick                        m_Nick;
 	unsigned int                 m_uJoinTries;
 	CString                      m_sDefaultModes;
-	map<CString,CNick>           m_msNicks;       // Todo: make this caseless (irc style)
-	unsigned int                 m_uBufferCount;
-	vector<CString>              m_vsBuffer;
+	std::map<CString,CNick>      m_msNicks;       // Todo: make this caseless (irc style)
+	CBuffer                      m_Buffer;
 
-	map<unsigned char, CString>  m_musModes;
+	bool                         m_bModeKnown;
+	std::map<unsigned char, CString> m_musModes;
 };
 
 #endif // !_CHAN_H
