@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2011  See the AUTHORS file for details.
+ * Copyright (C) 2004-2012  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,8 +9,8 @@
 #ifndef EXEC_SOCK_H
 #define EXEC_SOCK_H
 
-#include "zncconfig.h"
-#include "Socket.h"
+#include <znc/zncconfig.h>
+#include <znc/Socket.h>
 #include <signal.h>
 
 //! @author imaginos@imaginos.net
