@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2012  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,16 +9,15 @@
 #ifndef _FILEUTILS_H
 #define _FILEUTILS_H
 
-#include "Socket.h"
-#include "ZNCString.h"
+#include <znc/zncconfig.h>
+#include <znc/ZNCString.h>
 #include <dirent.h>
-#include <map>
-#include <signal.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/fcntl.h>
+#include <unistd.h>
 #include <vector>
 
-using std::vector;
-using std::map;
-
 class CFile {
 public:
 	CFile();
@@ -101,15 +100,15 @@ public:
 	bool Sync();
 	bool Open(const CString& sFileName, int iFlags = O_RDONLY, mode_t iMode = 0644);
 	bool Open(int iFlags = O_RDONLY, mode_t iMode = 0644);
-	int Read(char *pszBuffer, int iBytes);
+	ssize_t Read(char *pszBuffer, int iBytes);
 	bool ReadLine(CString & sData, const CString & sDelimiter = "\n");
 	bool ReadFile(CString& sData, size_t iMaxSize = 512 * 1024);
-	int Write(const char *pszBuffer, u_int iBytes);
-	int Write(const CString & sData);
+	ssize_t Write(const char *pszBuffer, size_t iBytes);
+	ssize_t Write(const CString & sData);
 	void Close();
 	void ClearBuffer();
 
-	bool TryExLock(const CString& sLockFile, int iFlags = O_RDONLY | O_CREAT);
+	bool TryExLock(const CString& sLockFile, int iFlags = O_RDWR | O_CREAT);
 	bool TryExLock();
 	bool ExLock();
 	bool UnLock();
@@ -119,19 +118,28 @@ public:
 	CString GetShortName() const;
 	CString GetDir() const;
 
+	bool HadError() const { return m_bHadError; }
+	void ResetError() { m_bHadError = false; }
+
+	static void InitHomePath(const CString& sFallback);
+	static const CString& GetHomePath() { return m_sHomePath; }
+
 private:
 	// fcntl() locking wrapper
-	bool Lock(int iType, bool bBlocking);
+	bool Lock(short iType, bool bBlocking);
 
 	CString m_sBuffer;
 	int     m_iFD;
+	bool    m_bHadError;
+
+	static CString m_sHomePath;
 
 protected:
 	CString m_sLongName;  //!< Absolute filename (m_sPath + "/" + m_sShortName)
 	CString m_sShortName; //!< Filename alone, without path
 };
 
-class CDir : public vector<CFile*> {
+class CDir : public std::vector<CFile*> {
 public:
 
 	CDir(const CString& sDir) {
@@ -157,11 +165,11 @@ public:
 		clear();
 	}
 
-	int Fill(const CString& sDir) {
+	size_t Fill(const CString& sDir) {
 		return FillByWildcard(sDir, "*");
 	}
 
-	int FillByWildcard(const CString& sDir, const CString& sWildcard) {
+	size_t FillByWildcard(const CString& sDir, const CString& sWildcard) {
 		CleanUp();
 		DIR* dir = opendir((sDir.empty()) ? "." : sDir.c_str());
 
@@ -246,37 +254,4 @@ protected:
 	CFile::EFileAttr m_eSortAttr;
 	bool             m_bDesc;
 };
-
-//! @author imaginos@imaginos.net
-class CExecSock : public CZNCSock {
-public:
-	CExecSock() : CZNCSock() {
-		m_iPid = -1;
-	}
-
-	int Execute(const CString & sExec) {
-		int iReadFD, iWriteFD;
-		m_iPid = popen2(iReadFD, iWriteFD, sExec);
-		if (m_iPid != -1) {
-			ConnectFD(iReadFD, iWriteFD, "0.0.0.0:0");
-		}
-		return(m_iPid);
-	}
-	void Kill(int iSignal)
-	{
-		kill(m_iPid, iSignal);
-		Close();
-	}
-	virtual ~CExecSock() {
-		close2(m_iPid, GetRSock(), GetWSock());
-		SetRSock(-1);
-		SetWSock(-1);
-	}
-
-	int popen2(int & iReadFD, int & iWriteFD, const CString & sCommand);
-	void close2(int iPid, int iReadFD, int iWriteFD);
-
-private:
-	int  m_iPid;
-};
 #endif // !_FILEUTILS_H
