@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2011  See the AUTHORS file for details.
+ * Copyright (C) 2004-2012  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,32 +9,29 @@
 #ifndef _MODULES_H
 #define _MODULES_H
 
-#include "zncconfig.h"
-#include "WebModules.h"
-#include "main.h"
+#include <znc/zncconfig.h>
+#include <znc/WebModules.h>
+#include <znc/main.h>
 #include <set>
 #include <queue>
 
-using std::set;
-
 // Forward Declarations
 class CAuthBase;
 class CChan;
+class CIRCNetwork;
 class CClient;
 class CWebSock;
 class CTemplate;
 class CIRCSock;
 class CModule;
-class CGlobalModule;
 class CModInfo;
 // !Forward Declarations
 
-// User Module Macros
 #ifdef REQUIRESSL
 #ifndef HAVE_LIBSSL
 #error -
 #error -
-#error This module only works when znc is compiled with OpenSSL support
+#error This module only works when ZNC is compiled with OpenSSL support
 #error -
 #error -
 #endif
@@ -45,12 +42,9 @@ typedef void* ModHandle;
 template<class M> void TModInfo(CModInfo& Info) {}
 
 template<class M> CModule* TModLoad(ModHandle p, CUser* pUser,
-		const CString& sModName, const CString& sModPath) {
-	return new M(p, pUser, sModName, sModPath);
-}
-template<class M> CGlobalModule* TModLoadGlobal(ModHandle p,
-		const CString& sModName, const CString& sModPath) {
-	return new M(p, sModName, sModPath);
+		CIRCNetwork* pNetwork, const CString& sModName,
+		const CString& sModPath) {
+	return new M(p, pUser, pNetwork, sModName, sModPath);
 }
 
 #if HAVE_VISIBILITY
@@ -59,15 +53,16 @@ template<class M> CGlobalModule* TModLoadGlobal(ModHandle p,
 # define MODULE_EXPORT
 #endif
 
-#define MODCOMMONDEFS(CLASS, DESCRIPTION, GLOBAL, LOADER) \
+#define MODCOMMONDEFS(CLASS, DESCRIPTION, TYPE) \
 	extern "C" { \
 		MODULE_EXPORT bool ZNCModInfo(double dCoreVersion, CModInfo& Info); \
 		bool ZNCModInfo(double dCoreVersion, CModInfo& Info) { \
 			if (dCoreVersion != VERSION) \
 				return false; \
 			Info.SetDescription(DESCRIPTION); \
-			Info.SetGlobal(GLOBAL); \
-			LOADER; \
+			Info.SetDefaultType(TYPE); \
+			Info.AddType(TYPE); \
+			Info.SetLoader(TModLoad<CLASS>); \
 			TModInfo<CLASS>(Info); \
 			return true; \
 		} \
@@ -87,34 +82,38 @@ template<class M> CGlobalModule* TModLoadGlobal(ModHandle p,
  *  \endcode
  *
  *  @param CLASS The name of your module's class.
- *  @see For global modules you need GLOBALMODCONSTRUCTOR.
  */
 #define MODCONSTRUCTOR(CLASS) \
-	CLASS(ModHandle pDLL, CUser* pUser, const CString& sModName, \
+	CLASS(ModHandle pDLL, CUser* pUser, CIRCNetwork* pNetwork, const CString& sModName, \
 			const CString& sModPath) \
-			: CModule(pDLL, pUser, sModName, sModPath)
+			: CModule(pDLL, pUser, pNetwork, sModName, sModPath)
 
-/** At the end of your source file, you must call this macro in global context.
- *  It defines some static functions which ZNC needs to load this module.
- *  @param CLASS The name of your module's class.
- *  @param DESCRIPTION A short description of your module.
- *  @see For global modules you need GLOBALMODULEDEFS.
- */
-#define MODULEDEFS(CLASS, DESCRIPTION) \
-	MODCOMMONDEFS(CLASS, DESCRIPTION, false, Info.SetLoader(TModLoad<CLASS>))
+// User Module Macros
+/** This works exactly like MODULEDEFS, but for user modules. */
+#define USERMODULEDEFS(CLASS, DESCRIPTION) \
+	MODCOMMONDEFS(CLASS, DESCRIPTION, CModInfo::UserModule)
 // !User Module Macros
 
 // Global Module Macros
-/** This works exactly like MODCONSTRUCTOR, but for global modules. */
-#define GLOBALMODCONSTRUCTOR(CLASS) \
-	CLASS(ModHandle pDLL, const CString& sModName, const CString& sModPath) \
-			: CGlobalModule(pDLL, sModName, sModPath)
-
 /** This works exactly like MODULEDEFS, but for global modules. */
 #define GLOBALMODULEDEFS(CLASS, DESCRIPTION) \
-	MODCOMMONDEFS(CLASS, DESCRIPTION, true, Info.SetGlobalLoader(TModLoadGlobal<CLASS>))
+	MODCOMMONDEFS(CLASS, DESCRIPTION, CModInfo::GlobalModule)
 // !Global Module Macros
 
+// Network Module Macros
+/** This works exactly like MODULEDEFS, but for network modules. */
+#define NETWORKMODULEDEFS(CLASS, DESCRIPTION) \
+	MODCOMMONDEFS(CLASS, DESCRIPTION, CModInfo::NetworkModule)
+// !Network Module Macros
+
+/** At the end of your source file, you must call this macro in global context.
+ *  It defines some static functions which ZNC needs to load this module.
+ *  By default the module will be a network module.
+ *  @param CLASS The name of your module's class.
+ *  @param DESCRIPTION A short description of your module.
+ */
+#define MODULEDEFS(CLASS, DESCRIPTION) NETWORKMODULEDEFS(CLASS, DESCRIPTION)
+
 // Forward Declarations
 class CZNC;
 class CUser;
@@ -172,19 +171,23 @@ private:
 
 class CModInfo {
 public:
-	typedef CModule* (*ModLoader)(ModHandle p, CUser* pUser, const CString& sModName, const CString& sModPath);
-	typedef CGlobalModule* (*GlobalModLoader)(ModHandle p, const CString& sModName, const CString& sModPath);
+	typedef CModule* (*ModLoader)(ModHandle p, CUser* pUser, CIRCNetwork* pNetwork, const CString& sModName, const CString& sModPath);
+
+	typedef enum {
+		GlobalModule,
+		UserModule,
+		NetworkModule
+	} EModuleType;
 
 	CModInfo() {
-		m_fGlobalLoader = NULL;
 		m_fLoader = NULL;
+		m_bHasArgs = false;
 	}
-	CModInfo(const CString& sName, const CString& sPath, bool bGlobal) {
-		m_bGlobal = bGlobal;
+	CModInfo(const CString& sName, const CString& sPath, EModuleType eType) {
 		m_sName = sName;
 		m_sPath = sPath;
-		m_fGlobalLoader = NULL;
 		m_fLoader = NULL;
+		m_bHasArgs = false;
 	}
 	~CModInfo() {}
 
@@ -192,14 +195,32 @@ public:
 		return (GetName() < Info.GetName());
 	}
 
+	bool SupportsType(EModuleType eType) {
+		return m_seType.find(eType) != m_seType.end();
+	}
+
+	void AddType(EModuleType eType) {
+		m_seType.insert(eType);
+	}
+
+	static CString ModuleTypeToString(EModuleType eType) {
+		switch (eType) {
+		case GlobalModule: return "Global";
+		case UserModule: return "User";
+		case NetworkModule: return "Network";
+		default: return "UNKNOWN";
+		}
+	}
+
 	// Getters
 	const CString& GetName() const { return m_sName; }
 	const CString& GetPath() const { return m_sPath; }
 	const CString& GetDescription() const { return m_sDescription; }
 	const CString& GetWikiPage() const { return m_sWikiPage; }
-	bool IsGlobal() const { return m_bGlobal; }
+	const CString& GetArgsHelpText() const { return m_sArgsHelpText; }
+	bool GetHasArgs() const { return m_bHasArgs; }
 	ModLoader GetLoader() const { return m_fLoader; }
-	GlobalModLoader GetGlobalLoader() const { return m_fGlobalLoader; }
+	EModuleType GetDefaultType() const { return m_eDefaultType; }
 	// !Getters
 
 	// Setters
@@ -207,19 +228,22 @@ public:
 	void SetPath(const CString& s) { m_sPath = s; }
 	void SetDescription(const CString& s) { m_sDescription = s; }
 	void SetWikiPage(const CString& s) { m_sWikiPage = s; }
-	void SetGlobal(bool b) { m_bGlobal = b; }
+	void SetArgsHelpText(const CString& s) { m_sArgsHelpText = s; }
+	void SetHasArgs(bool b = false) { m_bHasArgs = b; }
 	void SetLoader(ModLoader fLoader) { m_fLoader = fLoader; }
-	void SetGlobalLoader(GlobalModLoader fGlobalLoader) { m_fGlobalLoader = fGlobalLoader; }
+	void SetDefaultType(EModuleType eType) { m_eDefaultType = eType; }
 	// !Setters
 private:
 protected:
-	bool            m_bGlobal;
+	std::set<EModuleType> m_seType;
+	EModuleType     m_eDefaultType;
 	CString         m_sName;
 	CString         m_sPath;
 	CString         m_sDescription;
 	CString         m_sWikiPage;
+	CString         m_sArgsHelpText;
+	bool            m_bHasArgs;
 	ModLoader       m_fLoader;
-	GlobalModLoader m_fGlobalLoader;
 };
 
 /** A helper class for handling commands in modules. */
@@ -276,7 +300,7 @@ private:
 
 /** The base class for your own ZNC modules.
  *
- *  If you want to write a module for znc, you will have to implement a class
+ *  If you want to write a module for ZNC, you will have to implement a class
  *  which inherits from this class. You should override some of the "On*"
  *  functions in this class. These function will then be called by ZNC when the
  *  associated event happens.
@@ -289,9 +313,8 @@ private:
  */
 class CModule {
 public:
-	CModule(ModHandle pDLL, CUser* pUser, const CString& sModName,
+	CModule(ModHandle pDLL, CUser* pUser, CIRCNetwork* pNetwork, const CString& sModName,
 			const CString& sDataDir);
-	CModule(ModHandle pDLL, const CString& sModName, const CString& sDataDir);
 	virtual ~CModule();
 
 	/** This enum is just used for return from module hooks. Based on this
@@ -326,6 +349,7 @@ public:
 	} EModException;
 
 	void SetUser(CUser* pUser);
+	void SetNetwork(CIRCNetwork* pNetwork);
 	void SetClient(CClient* pClient);
 
 	/** This function throws CModule::UNLOAD which causes this module to be unloaded.
@@ -358,6 +382,8 @@ public:
 	 *  @return The Title.
 	 */
 	virtual CString GetWebMenuTitle() { return ""; }
+	virtual CString GetWebPath();
+	virtual CString GetWebFilesPath();
 	/** For WebMods: Called before the list of registered SubPages will be checked.
 	 *  Important: If you return true, you need to take care of calling WebSock.Close!
 	 *  This allows for stuff like returning non-templated data, long-polling and other fun.
@@ -511,14 +537,14 @@ public:
 	 *  @param sMessage The quit message.
 	 *  @param vChans List of channels which you and nick share.
 	 */
-	virtual void OnQuit(const CNick& Nick, const CString& sMessage, const vector<CChan*>& vChans);
+	virtual void OnQuit(const CNick& Nick, const CString& sMessage, const std::vector<CChan*>& vChans);
 	/** Called when a nickname change occurs. If we are changing our nick,
 	 *  sNewNick will equal m_pIRCSock->GetNick().
 	 *  @param Nick The nick which changed its nickname
 	 *  @param sNewNick The new nickname.
 	 *  @param vChans Channels which we and nick share.
 	 */
-	virtual void OnNick(const CNick& Nick, const CString& sNewNick, const vector<CChan*>& vChans);
+	virtual void OnNick(const CNick& Nick, const CString& sNewNick, const std::vector<CChan*>& vChans);
 	/** Called when a nick is kicked from a channel.
 	 *  @param OpNick The nick which generated the kick.
 	 *  @param sKickedNick The nick which was kicked.
@@ -537,6 +563,12 @@ public:
 	 *  @param sMessage The part message.
 	 */
 	virtual void OnPart(const CNick& Nick, CChan& Channel, const CString& sMessage);
+	/** Called when user is invited into a channel
+	 *  @param Nick The nick who invited you.
+	 *  @param sChan The channel the user got invited into
+	 *  @return See CModule::EModRet.
+	 */
+	virtual EModRet OnInvite(const CNick& Nick, const CString& sChan);
 
 	/** Called before a channel buffer is played back to a client.
 	 *  @param Chan The channel which will be played back.
@@ -790,8 +822,8 @@ public:
 	bool RemTimer(const CString& sLabel);
 	bool UnlinkTimer(CTimer* pTimer);
 	CTimer* FindTimer(const CString& sLabel);
-	set<CTimer*>::const_iterator BeginTimers() const { return m_sTimers.begin(); }
-	set<CTimer*>::const_iterator EndTimers() const { return m_sTimers.end(); }
+	std::set<CTimer*>::const_iterator BeginTimers() const { return m_sTimers.begin(); }
+	std::set<CTimer*>::const_iterator EndTimers() const { return m_sTimers.end(); }
 	virtual void ListTimers();
 	// !Timer stuff
 
@@ -801,8 +833,8 @@ public:
 	bool RemSocket(const CString& sSockName);
 	bool UnlinkSocket(CSocket* pSocket);
 	CSocket* FindSocket(const CString& sSockName);
-	set<CSocket*>::const_iterator BeginSockets() const { return m_sSockets.begin(); }
-	set<CSocket*>::const_iterator EndSockets() const { return m_sSockets.end(); }
+	std::set<CSocket*>::const_iterator BeginSockets() const { return m_sSockets.begin(); }
+	std::set<CSocket*>::const_iterator EndSockets() const { return m_sSockets.end(); }
 	virtual void ListSockets();
 	// !Socket stuff
 
@@ -845,14 +877,14 @@ public:
 	const CString& GetSavePath() const;
 
 	// Setters
-	void SetGlobal(bool b) { m_bGlobal = b; }
+	void SetType(CModInfo::EModuleType eType) { m_eType = eType; }
 	void SetDescription(const CString& s) { m_sDescription = s; }
 	void SetModPath(const CString& s) { m_sModPath = s; }
 	void SetArgs(const CString& s) { m_sArgs = s; }
 	// !Setters
 
 	// Getters
-	bool IsGlobal() const { return m_bGlobal; }
+	CModInfo::EModuleType GetType() const { return m_eType; }
 	const CString& GetDescription() const { return m_sDescription; }
 	const CString& GetArgs() const { return m_sArgs; }
 	const CString& GetModPath() const { return m_sModPath; }
@@ -866,18 +898,115 @@ public:
 	/** @returns NULL except when we are in a client-specific module hook in
 	 *           which case this is the client for which the hook is called.
 	 */
+	CIRCNetwork* GetNetwork() { return m_pNetwork; }
 	CClient* GetClient() { return m_pClient; }
 	CSockManager* GetManager() { return m_pManager; }
 	// !Getters
 
+	// Global Modules
+	/** This module hook is called when a user is being added.
+	 * @param User The user which will be added.
+	 * @param sErrorRet A message that may be displayed to the user if
+	 *                  the module stops adding the user.
+	 * @return See CModule::EModRet.
+	 */
+	virtual EModRet OnAddUser(CUser& User, CString& sErrorRet);
+	/** This module hook is called when a user is deleted.
+	 *  @param User The user which will be deleted.
+	 *  @return See CModule::EModRet.
+	 */
+	virtual EModRet OnDeleteUser(CUser& User);
+	/** This module hook is called when there is an incoming connection on
+	 *  any of ZNC's listening sockets.
+	 *  @param pSock The incoming client socket.
+	 *  @param sHost The IP the client is connecting from.
+	 *  @param uPort The port the client is connecting from.
+	 */
+	virtual void OnClientConnect(CZNCSock* pSock, const CString& sHost, unsigned short uPort);
+	/** This module hook is called when a client tries to login. If your
+	 *  module wants to handle the login attempt, it must return
+	 *  CModule::EModRet::HALT;
+	 *  @param Auth The necessary authentication info for this login attempt.
+	 *  @return See CModule::EModRet.
+	 */
+	virtual EModRet OnLoginAttempt(CSmartPtr<CAuthBase> Auth);
+	/** Called after a client login was rejected.
+	 *  @param sUsername The username that tried to log in.
+	 *  @param sRemoteIP The IP address from which the client tried to login.
+	 */
+	virtual void OnFailedLogin(const CString& sUsername, const CString& sRemoteIP);
+	/** This function behaves like CModule::OnRaw(), but is also called
+	 *  before the client successfully logged in to ZNC. You should always
+	 *  prefer to use CModule::OnRaw() if possible.
+	 *  @param pClient The client which send this line.
+	 *  @param sLine The raw traffic line which the client sent.
+	 */
+	virtual EModRet OnUnknownUserRaw(CClient* pClient, CString& sLine);
+
+	/** Called when a client told us CAP LS. Use ssCaps.insert("cap-name")
+	 *  for announcing capabilities which your module supports.
+	 *  @param pClient The client which requested the list.
+	 *  @param ssCaps set of caps which will be sent to client.
+	 */
+	virtual void OnClientCapLs(CClient* pClient, SCString& ssCaps);
+	/** Called only to check if your module supports turning on/off named capability.
+	 *  @param pClient The client which wants to enable/disable a capability.
+	 *  @param sCap name of capability.
+	 *  @param bState On or off, depending on which case is interesting for client.
+	 *  @return true if your module supports this capability in the specified state.
+	 */
+	virtual bool IsClientCapSupported(CClient* pClient, const CString& sCap, bool bState);
+	/** Called when we actually need to turn a capability on or off for a client.
+	 *  @param pClient The client which requested the capability.
+	 *  @param sCap name of wanted capability.
+	 *  @param bState On or off, depending on which case client needs.
+	 */
+	virtual void OnClientCapRequest(CClient* pClient, const CString& sCap, bool bState);
+
+	/** Called when a module is going to be loaded.
+	 *  @param sModName name of the module.
+	 *  @param eType wanted type of the module (user/global).
+	 *  @param sArgs arguments of the module.
+	 *  @param[out] bSuccess the module was loaded successfully
+	 *                       as result of this module hook?
+	 *  @param[out] sRetMsg text about loading of the module.
+	 *  @return See CModule::EModRet.
+	 */
+	virtual EModRet OnModuleLoading(const CString& sModName, const CString& sArgs,
+			CModInfo::EModuleType eType, bool& bSuccess, CString& sRetMsg);
+	/** Called when a module is going to be unloaded.
+	 *  @param pModule the module.
+	 *  @param[out] bSuccess the module was unloaded successfully
+	 *                       as result of this module hook?
+	 *  @param[out] sRetMsg text about unloading of the module.
+	 *  @return See CModule::EModRet.
+	 */
+	virtual EModRet OnModuleUnloading(CModule* pModule, bool& bSuccess, CString& sRetMsg);
+	/** Called when info about a module is needed.
+	 *  @param[out] ModInfo put result here, if your module knows it.
+	 *  @param sModule name of the module.
+	 *  @param bSuccess this module provided info about the module.
+	 *  @param sRetMsg text describing possible issues.
+	 *  @return See CModule::EModRet.
+	 */
+	virtual EModRet OnGetModInfo(CModInfo& ModInfo, const CString& sModule,
+			bool& bSuccess, CString& sRetMsg);
+	/** Called when list of available mods is requested.
+	 *  @param ssMods put new modules here.
+	 *  @param bGlobal true if global modules are needed.
+	 */
+	virtual void OnGetAvailableMods(std::set<CModInfo>& ssMods, CModInfo::EModuleType eType);
+	// !Global Modules
+
 protected:
-	bool               m_bGlobal;
+	CModInfo::EModuleType m_eType;
 	CString            m_sDescription;
-	set<CTimer*>       m_sTimers;
-	set<CSocket*>      m_sSockets;
+	std::set<CTimer*>  m_sTimers;
+	std::set<CSocket*> m_sSockets;
 	ModHandle          m_pDLL;
 	CSockManager*      m_pManager;
 	CUser*             m_pUser;
+	CIRCNetwork*       m_pNetwork;
 	CClient*           m_pClient;
 	CString            m_sModName;
 	CString            m_sDataDir;
@@ -887,17 +1016,19 @@ protected:
 private:
 	MCString           m_mssRegistry; //!< way to save name/value pairs. Note there is no encryption involved in this
 	VWebSubPages       m_vSubPages;
-	map<CString, CModCommand> m_mCommands;
+	std::map<CString, CModCommand> m_mCommands;
 };
 
-class CModules : public vector<CModule*> {
+class CModules : public std::vector<CModule*> {
 public:
 	CModules();
 	~CModules();
 
 	void SetUser(CUser* pUser) { m_pUser = pUser; }
+	void SetNetwork(CIRCNetwork* pNetwork) { m_pNetwork = pNetwork; }
 	void SetClient(CClient* pClient) { m_pClient = pClient; }
 	CUser* GetUser() { return m_pUser; }
+	CIRCNetwork* GetNetwork() { return m_pNetwork; }
 	CClient* GetClient() { return m_pClient; }
 
 	void UnloadAll();
@@ -927,11 +1058,12 @@ public:
 	bool OnModNotice(const CString& sMessage);
 	bool OnModCTCP(const CString& sMessage);
 
-	bool OnQuit(const CNick& Nick, const CString& sMessage, const vector<CChan*>& vChans);
-	bool OnNick(const CNick& Nick, const CString& sNewNick, const vector<CChan*>& vChans);
+	bool OnQuit(const CNick& Nick, const CString& sMessage, const std::vector<CChan*>& vChans);
+	bool OnNick(const CNick& Nick, const CString& sNewNick, const std::vector<CChan*>& vChans);
 	bool OnKick(const CNick& Nick, const CString& sOpNick, CChan& Channel, const CString& sMessage);
 	bool OnJoin(const CNick& Nick, CChan& Channel);
 	bool OnPart(const CNick& Nick, CChan& Channel, const CString& sMessage);
+	bool OnInvite(const CNick& Nick, const CString& sChan);
 
 	bool OnChanBufferStarting(CChan& Chan, CClient& Client);
 	bool OnChanBufferEnding(CChan& Chan, CClient& Client);
@@ -967,14 +1099,14 @@ public:
 	bool OnServerCapResult(const CString& sCap, bool bSuccess);
 
 	CModule* FindModule(const CString& sModule) const;
-	bool LoadModule(const CString& sModule, const CString& sArgs, CUser* pUser, CString& sRetMsg);
+	bool LoadModule(const CString& sModule, const CString& sArgs, CModInfo::EModuleType eType, CUser* pUser, CIRCNetwork* pNetwork, CString& sRetMsg);
 	bool UnloadModule(const CString& sModule);
 	bool UnloadModule(const CString& sModule, CString& sRetMsg);
-	bool ReloadModule(const CString& sModule, const CString& sArgs, CUser* pUser, CString& sRetMsg);
+	bool ReloadModule(const CString& sModule, const CString& sArgs, CUser* pUser, CIRCNetwork* pNetwork, CString& sRetMsg);
 
 	static bool GetModInfo(CModInfo& ModInfo, const CString& sModule, CString &sRetMsg);
 	static bool GetModPathInfo(CModInfo& ModInfo, const CString& sModule, const CString& sModPath, CString &sRetMsg);
-	static void GetAvailableMods(set<CModInfo>& ssMods, bool bGlobal = false);
+	static void GetAvailableMods(std::set<CModInfo>& ssMods, CModInfo::EModuleType eType = CModInfo::UserModule);
 
 	// This returns the path to the .so and to the data dir
 	// which is where static data (webadmin skins) are saved
@@ -985,143 +1117,32 @@ public:
 	typedef std::queue<std::pair<CString, CString> > ModDirList;
 	static ModDirList GetModDirs();
 
-private:
-	static ModHandle OpenModule(const CString& sModule, const CString& sModPath,
-			bool &bVersionMismatch, CModInfo& Info, CString& sRetMsg);
-
-protected:
-	CUser*    m_pUser;
-	CClient*  m_pClient;
-};
-
-/** Base class for global modules. If you want to write a global module, your
- *  module class has to derive from CGlobalModule instead of CModule.
- *
- *  All the module hooks from CModule work here, too. The difference is that
- *  they are now called for all users instead of just a specific one.
- *
- *  Instead of MODCONSTRUCTOR and MODULEDEFS, you will have to use
- *  GLOBALMODCONSTRUCTOR and GLOBALMODULEDEFS.
- */
-class CGlobalModule : public CModule {
-public:
-	CGlobalModule(ModHandle pDLL, const CString& sModName,
-			const CString &sDataDir) : CModule(pDLL, sModName, sDataDir) {}
-	virtual ~CGlobalModule() {}
-
-	/** This module hook is called when a user is being added.
-	 * @param User The user which will be added.
-	 * @param sErrorRet A message that may be displayed to the user if
-	 *                  the module stops adding the user.
-	 * @return See CModule::EModRet.
-	 */
-	virtual EModRet OnAddUser(CUser& User, CString& sErrorRet);
-	/** This module hook is called when a user is deleted.
-	 *  @param User The user which will be deleted.
-	 *  @return See CModule::EModRet.
-	 */
-	virtual EModRet OnDeleteUser(CUser& User);
-	/** This module hook is called when there is an incoming connection on
-	 *  any of ZNC's listening sockets.
-	 *  @param pSock The incoming client socket.
-	 *  @param sHost The IP the client is connecting from.
-	 *  @param uPort The port the client is connecting from.
-	 */
-	virtual void OnClientConnect(CZNCSock* pSock, const CString& sHost, unsigned short uPort);
-	/** This module hook is called when a client tries to login. If your
-	 *  module wants to handle the login attempt, it must return
-	 *  CModule::EModRet::HALT;
-	 *  @param Auth The necessary authentication info for this login attempt.
-	 *  @return See CModule::EModRet.
-	 */
-	virtual EModRet OnLoginAttempt(CSmartPtr<CAuthBase> Auth);
-	/** Called after a client login was rejected.
-	 *  @param sUsername The username that tried to log in.
-	 *  @param sRemoteIP The IP address from which the client tried to login.
-	 */
-	virtual void OnFailedLogin(const CString& sUsername, const CString& sRemoteIP);
-	/** This function behaves like CModule::OnRaw(), but is also called
-	 *  before the client successfully logged in to ZNC. You should always
-	 *  prefer to use CModule::OnRaw() if possible.
-	 *  @param sLine The raw traffic line which the client sent.
-	 *  @todo Why doesn't this use m_pUser and m_pClient?
-	 *        (Well, ok, m_pUser isn't known yet...)
-	 */
-	virtual EModRet OnUnknownUserRaw(CString& sLine);
-
-	/** Called when a client told us CAP LS. Use ssCaps.insert("cap-name")
-	 *  for announcing capabilities which your module supports.
-	 *  @param ssCaps set of caps which will be sent to client.
-	 */
-	virtual void OnClientCapLs(SCString& ssCaps);
-	/** Called only to check if your module supports turning on/off named capability.
-	 *  @param sCap name of capability.
-	 *  @param bState On or off, depending on which case is interesting for client.
-	 *  @return true if your module supports this capability in the specified state.
-	 */
-	virtual bool IsClientCapSupported(const CString& sCap, bool bState);
-	/** Called when we actually need to turn a capability on or off for a client.
-	 *  @param sCap name of wanted capability.
-	 *  @param bState On or off, depending on which case client needs.
-	 */
-	virtual void OnClientCapRequest(const CString& sCap, bool bState);
-
-	/** Called when a module is going to be loaded.
-	 *  @param sModName name of the module.
-	 *  @param sArgs arguments of the module.
-	 *  @param[out] bSuccess the module was loaded successfully
-	 *                       as result of this module hook?
-	 *  @param[out] sRetMsg text about loading of the module.
-	 *  @return See CModule::EModRet.
-	 */
-	virtual EModRet OnModuleLoading(const CString& sModName, const CString& sArgs,
-			bool& bSuccess, CString& sRetMsg);
-	/** Called when a module is going to be unloaded.
-	 *  @param pModule the module.
-	 *  @param[out] bSuccess the module was unloaded successfully
-	 *                       as result of this module hook?
-	 *  @param[out] sRetMsg text about unloading of the module.
-	 *  @return See CModule::EModRet.
-	 */
-	virtual EModRet OnModuleUnloading(CModule* pModule, bool& bSuccess, CString& sRetMsg);
-	/** Called when info about a module is needed.
-	 *  @param[out] ModInfo put result here, if your module knows it.
-	 *  @param sModule name of the module.
-	 *  @param bSuccess this module provided info about the module.
-	 *  @param sRetMsg text describing possible issues.
-	 *  @return See CModule::EModRet.
-	 */
-	virtual EModRet OnGetModInfo(CModInfo& ModInfo, const CString& sModule,
-			bool& bSuccess, CString& sRetMsg);
-	/** Called when list of available mods is requested.
-	 *  @param ssMods put new modules here.
-	 *  @param bGlobal true if global modules are needed.
-	 */
-	virtual void OnGetAvailableMods(set<CModInfo>& ssMods, bool bGlobal);
-private:
-};
-
-class CGlobalModules : public CModules {
-public:
-	CGlobalModules() : CModules() {}
-	~CGlobalModules() {}
-
+	// Global Modules
 	bool OnAddUser(CUser& User, CString& sErrorRet);
 	bool OnDeleteUser(CUser& User);
 	bool OnClientConnect(CZNCSock* pSock, const CString& sHost, unsigned short uPort);
 	bool OnLoginAttempt(CSmartPtr<CAuthBase> Auth);
 	bool OnFailedLogin(const CString& sUsername, const CString& sRemoteIP);
-	bool OnUnknownUserRaw(CString& sLine);
-	bool OnClientCapLs(SCString& ssCaps);
-	bool IsClientCapSupported(const CString& sCap, bool bState);
-	bool OnClientCapRequest(const CString& sCap, bool bState);
+	bool OnUnknownUserRaw(CClient* pClient, CString& sLine);
+	bool OnClientCapLs(CClient* pClient, SCString& ssCaps);
+	bool IsClientCapSupported(CClient* pClient, const CString& sCap, bool bState);
+	bool OnClientCapRequest(CClient* pClient, const CString& sCap, bool bState);
 	bool OnModuleLoading(const CString& sModName, const CString& sArgs,
-			bool& bSuccess, CString& sRetMsg);
+			CModInfo::EModuleType eType, bool& bSuccess, CString& sRetMsg);
 	bool OnModuleUnloading(CModule* pModule, bool& bSuccess, CString& sRetMsg);
 	bool OnGetModInfo(CModInfo& ModInfo, const CString& sModule,
 			bool& bSuccess, CString& sRetMsg);
-	bool OnGetAvailableMods(set<CModInfo>& ssMods, bool bGlobal);
+	bool OnGetAvailableMods(std::set<CModInfo>& ssMods, CModInfo::EModuleType eType);
+	// !Global Modules
+
 private:
+	static ModHandle OpenModule(const CString& sModule, const CString& sModPath,
+			bool &bVersionMismatch, CModInfo& Info, CString& sRetMsg);
+
+protected:
+	CUser*        m_pUser;
+	CIRCNetwork*  m_pNetwork;
+	CClient*      m_pClient;
 };
 
 #endif // !_MODULES_H
