@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2011  See the AUTHORS file for details.
+ * Copyright (C) 2004-2012  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,11 +9,13 @@
 #ifndef ZNCDEBUG_H
 #define ZNCDEBUG_H
 
-#include "zncconfig.h"
+#include <znc/zncconfig.h>
+#include <znc/ZNCString.h>
 #include <iostream>
-
-using std::cout;
-using std::endl;
+#include <ctime>
+#include <sys/time.h>
+#include <sstream>
+#include <iomanip>
 
 /** Output a debug info if debugging is enabled.
  *  If ZNC was compiled with <code>--enable-debug</code> or was started with
@@ -28,7 +30,7 @@ using std::endl;
  */
 #define DEBUG(f) do { \
 	if (CDebug::Debug()) { \
-		cout << f << endl; \
+		std::cout << CDebug::GetTimestamp() << f << std::endl; \
 	} \
 } while (0)
 
@@ -38,6 +40,17 @@ public:
 	static bool StdoutIsTTY() { return stdoutIsTTY; }
 	static void SetDebug(bool b) { debug = b; }
 	static bool Debug() { return debug; }
+	static CString GetTimestamp() {
+		char buf[64];
+		timeval time_now;
+		gettimeofday(&time_now, NULL);
+		time_t currentSec = (time_t) time_now.tv_sec; // cast from long int
+		tm* time_info = localtime(&currentSec);
+		strftime(buf, sizeof(buf), "[%Y-%m-%d %H:%M:%S.", time_info);
+		std::ostringstream buffer;
+		buffer << buf << std::setw(6) << std::setfill('0') << (time_now.tv_usec) << "] ";
+		return buffer.str();
+	}
 
 protected:
 	static bool stdoutIsTTY;
