@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2011  See the AUTHORS file for details.
+ * Copyright (C) 2004-2012  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,31 +9,24 @@
 #ifndef ZNCSTRING_H
 #define ZNCSTRING_H
 
-#include "zncconfig.h"
+#include <znc/zncconfig.h>
 #include <map>
 #include <set>
 #include <string>
 #include <vector>
-#include <list>
 #include <sys/types.h>
 
-using std::map;
-using std::set;
-using std::string;
-using std::vector;
-using std::list;
-using std::pair;
-
 #define _SQL(s) CString("'" + CString(s).Escape_n(CString::ESQL) + "'")
 #define _URL(s) CString(s).Escape_n(CString::EURL)
 #define _HTML(s) CString(s).Escape_n(CString::EHTML)
+#define _NAMEDFMT(s) CString(s).Escape_n(CString::ENAMEDFMT)
 
 class CString;
 class MCString;
 
-typedef set<CString> SCString;
-typedef vector<CString>                 VCString;
-typedef vector<pair<CString, CString> > VPair;
+typedef std::set<CString> SCString;
+typedef std::vector<CString>                 VCString;
+typedef std::vector<std::pair<CString, CString> > VPair;
 
 static const unsigned char XX = 0xff;
 static const unsigned char base64_table[256] = {
@@ -56,22 +49,23 @@ static const unsigned char base64_table[256] = {
 };
 
 /**
- * @brief String class that is used inside znc.
+ * @brief String class that is used inside ZNC.
  *
  * All strings that are used in ZNC and its modules should use instances of this
  * class. It provides helpful functions for parsing input like Token() and
  * Split().
  */
-class CString : public string {
+class CString : public std::string {
 public:
 	typedef enum {
 		EASCII,
 		EURL,
 		EHTML,
-		ESQL
+		ESQL,
+		ENAMEDFMT
 	} EEscape;
 
-	explicit CString(bool b) : string(b ? "true" : "false") {}
+	explicit CString(bool b) : std::string(b ? "true" : "false") {}
 	explicit CString(char c);
 	explicit CString(unsigned char c);
 	explicit CString(short i);
@@ -85,10 +79,11 @@ public:
 	explicit CString(double i, int precision = 2);
 	explicit CString(float i, int precision = 2);
 
-	CString() : string() {}
-	CString(const char* c) : string(c) {}
-	CString(const char* c, size_t l) : string(c, l) {}
-	CString(const string& s) : string(s) {}
+	CString() : std::string() {}
+	CString(const char* c) : std::string(c) {}
+	CString(const char* c, size_t l) : std::string(c, l) {}
+	CString(const std::string& s) : std::string(s) {}
+	CString(size_t n, char c) : std::string(n, c) {}
 	~CString() {}
 
 	/**
@@ -215,12 +210,12 @@ public:
 	 * @param uCount The number of characters to keep.
 	 * @return The resulting string.
 	 */
-	CString Left(unsigned int uCount) const;
+	CString Left(size_type uCount) const;
 	/** Return the right part of the string.
 	 * @param uCount The number of characters to keep.
 	 * @return The resulting string.
 	 */
-	CString Right(unsigned int uCount) const;
+	CString Right(size_type uCount) const;
 
 	/** Get the first line of this string.
 	 * @return The first line of text.
@@ -242,17 +237,17 @@ public:
 	 *         after it.
 	 * @see Split() if you need a string split into all of its tokens.
 	 */
-	CString Token(unsigned int uPos, bool bRest = false, const CString& sSep = " ", bool bAllowEmpty = false) const;
+	CString Token(size_t uPos, bool bRest = false, const CString& sSep = " ", bool bAllowEmpty = false) const;
 
 	/** Get a token out of this string. This function behaves much like the
 	 *  other Token() function in this class. The extra arguments are
 	 *  handled similarly to Split().
 	 */
-	CString Token(unsigned int uPos, bool bRest, const CString& sSep, bool bAllowEmpty, const CString& sLeft, const CString& sRight, bool bTrimQuotes = true) const;
+	CString Token(size_t uPos, bool bRest, const CString& sSep, bool bAllowEmpty, const CString& sLeft, const CString& sRight, bool bTrimQuotes = true) const;
 
-	unsigned int URLSplit(MCString& msRet) const;
-	unsigned int OptionSplit(MCString& msRet, bool bUpperKeys = false) const;
-	unsigned int QuoteSplit(VCString& vsRet) const;
+	size_type URLSplit(MCString& msRet) const;
+	size_type OptionSplit(MCString& msRet, bool bUpperKeys = false) const;
+	size_type QuoteSplit(VCString& vsRet) const;
 
 	/** Split up this string into tokens.
 	 * Via sLeft and sRight you can define "markers" like with Replace().
@@ -269,7 +264,7 @@ public:
 	 *                        each token.
 	 * @return The number of tokens found.
 	 */
-	unsigned int Split(const CString& sDelim, VCString& vsRet, bool bAllowEmpty = true,
+	size_type Split(const CString& sDelim, VCString& vsRet, bool bAllowEmpty = true,
 					   const CString& sLeft = "", const CString& sRight = "", bool bTrimQuotes = true,
 					   bool bTrimWhiteSpace = false) const;
 
@@ -277,10 +272,20 @@ public:
 	 * This function is identical to the other CString::Split(), except that
 	 * the result is returned as a SCString instead of a VCString.
 	 */
-	unsigned int Split(const CString& sDelim, SCString& ssRet, bool bAllowEmpty = true,
+	size_type Split(const CString& sDelim, SCString& ssRet, bool bAllowEmpty = true,
 					   const CString& sLeft = "", const CString& sRight = "", bool bTrimQuotes = true,
 					   bool bTrimWhiteSpace = false) const;
 
+	/** Build a string from a format string, replacing values from a map.
+	 * The format specification can contain simple named parameters that match
+	 * keys in the given map. For example in the string "a {b} c", the key "b"
+	 * is looked up in the map, and inserted for "{b}".
+	 * @param sFormat The format specification.
+	 * @param msValues A map of named parameters to their values.
+	 * @return The string with named parameters replaced.
+	 */
+	static CString NamedFormat(const CString& sFormat, const MCString& msValues);
+
 	/** Produces a random string.
 	 * @param uLength The length of the resulting string.
 	 * @return A random string.
@@ -412,7 +417,7 @@ public:
 	 * @param sPrefix The prefix that should be removed.
 	 * @return True if this string was modified.
 	 */
-	bool TrimPrefix(const CString& sPrefix);
+	bool TrimPrefix(const CString& sPrefix = ":");
 	/** Trim a given suffix.
 	 * @param sSuffix The suffix that should be removed.
 	 * @return True if this string was modified.
@@ -422,7 +427,7 @@ public:
 	 * @param sPrefix The prefix that should be removed.
 	 * @return A copy of this string without the prefix.
 	 */
-	CString TrimPrefix_n(const CString& sPrefix) const;
+	CString TrimPrefix_n(const CString& sPrefix = ":") const;
 	/** Trim a given suffix.
 	 * @param sSuffix The suffix that should be removed.
 	 * @return A copy of this string without the prefix.
@@ -433,24 +438,24 @@ public:
 	 * @param uLen The number of characters to remove.
 	 * @return true if this string was modified.
 	 */
-	bool LeftChomp(unsigned int uLen = 1);
+	bool LeftChomp(size_type uLen = 1);
 	/** Remove characters from the end of this string.
 	 * @param uLen The number of characters to remove.
 	 * @return true if this string was modified.
 	 */
-	bool RightChomp(unsigned int uLen = 1);
+	bool RightChomp(size_type uLen = 1);
 	/** Remove characters from the beginning of this string.
 	 * This string object isn't modified.
 	 * @param uLen The number of characters to remove.
 	 * @return The result of the conversion.
 	 */
-	CString LeftChomp_n(unsigned int uLen = 1) const;
+	CString LeftChomp_n(size_type uLen = 1) const;
 	/** Remove characters from the end of this string.
 	 * This string object isn't modified.
 	 * @param uLen The number of characters to remove.
 	 * @return The result of the conversion.
 	 */
-	CString RightChomp_n(unsigned int uLen = 1) const;
+	CString RightChomp_n(size_type uLen = 1) const;
 
 private:
 protected:
@@ -462,13 +467,16 @@ protected:
  *
  * This class maps strings to other strings.
  */
-class MCString : public map<CString, CString> {
+class MCString : public std::map<CString, CString> {
 public:
 	/** Construct an empty MCString. */
-	MCString() : map<CString, CString>() {}
+	MCString() : std::map<CString, CString>() {}
 	/** Destruct this MCString. */
 	virtual ~MCString() { clear(); }
 
+	/** A static instance of an empty map. */
+	static const MCString EmptyMap;
+
 	/** Status codes that can be returned by WriteToDisk() and
 	 * ReadFromDisk(). */
 	enum status_t
