@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2010  See the AUTHORS file for details.
+ * Copyright (C) 2004-2012  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -9,10 +9,12 @@
 #ifndef _DEFINES_H
 #define _DEFINES_H
 
+#include <znc/zncconfig.h>
+
 // This header file is just for Csocket
 
-#include "main.h"
-#include "Utils.h"
+#include <znc/ZNCDebug.h>
+#include <znc/ZNCString.h>
 
 #define CS_STRING CString
 #define _NO_CSOCKET_NS
@@ -21,7 +23,7 @@
 #define __DEBUG__
 #endif
 
-// Redefine some Csocket debugging mechanisms to use znc's
+// Redefine some Csocket debugging mechanisms to use ZNC's
 #define CS_DEBUG(f)  DEBUG(__FILE__ << ":" << __LINE__ << " " << f)
 #define PERROR(f)    DEBUG(__FILE__ << ":" << __LINE__ << " " << f << ": " << strerror(GetSockError()))
 
